// Copyright (C) 2014 ChaosForge Ltd
// http://chaosforge.org/
//
// This file is part of NV Libraries.
// For conditions of distribution and use, see copyright notice in nv.hh

#ifndef NV_MESH_CREATOR_HH
#define NV_MESH_CREATOR_HH

#include <nv/core/common.hh>
#include <nv/core/math.hh>
#include <nv/interface/mesh_data.hh>

namespace nv
{

	class mesh_data_creator
	{
	public:
		mesh_data_creator( mesh_data* data ) : m_data( data ) {}
		// assumes that position and normal is vec3, tangent is vec4
		void transform( float scale, const mat3& r33 );
		// TODO: this could generate normals too
		void generate_tangents();
		void flip_normals();
		bool is_same_format( mesh_data* other );
		void merge( mesh_data* other );
	private:
		mesh_raw_channel* merge_channels( mesh_raw_channel* a, mesh_raw_channel* b );
		mesh_raw_channel* append_channels( mesh_raw_channel* a, mesh_raw_channel* b, uint32 frame_count = 1 );

		mesh_data* m_data;
	};

	class mesh_nodes_creator
	{
	public:
		mesh_nodes_creator( mesh_nodes_data* data ) : m_data( data ) {}
		// assumes that keys are equally spaced
		void pre_transform_keys();
		// assumes that keys are equally spaced
		void merge_keys();
		void transform( float scale, const mat3& r33 );
	private:
		mesh_nodes_data* m_data;
	};

	class mesh_creator
	{
	public:
		mesh_creator( mesh_data_pack* pack ) : m_pack( pack ) {}
		void delete_mesh( uint32 index )
		{
			if ( index < m_pack->get_count() )
			{
				m_pack->m_meshes[ index ].destroy();
				m_pack->m_meshes[ m_pack->m_count-1 ].move_to( m_pack->m_meshes[ index ] );
				m_pack->m_count--;
			}
		}
		// assumes that keys are equally spaced
		void pre_transform_keys() 
		{
			if ( m_pack->m_nodes )
				mesh_nodes_creator( m_pack->m_nodes ).pre_transform_keys();
		}
		void generate_tangents()
		{
			for ( size_t m = 0; m < m_pack->m_count; ++m )
				mesh_data_creator( &(m_pack->m_meshes[m]) ).generate_tangents();
		}

		// assumes that keys are equally spaced
		void merge_keys() { mesh_nodes_creator( m_pack->m_nodes ).merge_keys(); }
		// assumes that position and normal is vec3, tangent is vec4
		void flip_normals() 
		{
			for ( size_t m = 0; m < m_pack->m_count; ++m )
				mesh_data_creator( &(m_pack->m_meshes[m]) ).flip_normals();
		}
		void transform( float scale, const mat3& r33 ) 
		{
			for ( size_t m = 0; m < m_pack->m_count; ++m )
				mesh_data_creator( &(m_pack->m_meshes[m]) ).transform( scale, r33 );
			if ( m_pack->m_nodes )
				mesh_nodes_creator( m_pack->m_nodes ).transform( scale, r33 );
		}
		// assumes models have same format
		// currently only data merge
		bool merge( mesh_data_pack* other )
		{
			for ( size_t m = 0; m < m_pack->m_count; ++m )
			{
				if (!mesh_data_creator( &(m_pack->m_meshes[m]) ).is_same_format( &(other->m_meshes[m]) ) ) 
					return false;
			}
			for ( size_t m = 0; m < m_pack->m_count; ++m )
			{
				mesh_data_creator(&(m_pack->m_meshes[m]) ).merge( &(other->m_meshes[m]) );
			}
			return true;
		}
	private:
		mesh_data_pack* m_pack;
	};
}

#endif // NV_MESH_CREATOR_HH
